%define _disable_ld_no_undefined 0

# clang header paths are hard-coded at compile time
# and need adjustment whenever there's a new GCC version
%define gcc_version %(gcc -dumpversion)

%define compile_apidox 0
%{?_with_apidox: %{expand: %%global compile_apidox 1}}

%bcond_without clang

Name: llvm
Version: 3.1
Release: %mkrel 2
Summary: Low Level Virtual Machine (LLVM)
License: NCSA
Group: Development/Other
URL: http://llvm.org/
Source0: http://llvm.org/releases/%{version}/llvm-%{version}.src.tar.gz
Source1: http://llvm.org/releases/%{version}/clang-%{version}.src.tar.gz
# Versionize libclang.so (Anssi 08/2012):
Patch0: clang-soname.patch
# Add libclangTooling.a to libclang.so, backport from upstream
Patch1: clang-shared-tooling.patch
Obsoletes: llvm-ocaml
Requires: libstdc++-devel
BuildRequires: bison
BuildRequires: groff
BuildRequires: chrpath
BuildRequires: ocaml
BuildRequires: tcl
%if %{compile_apidox}
BuildRequires: doxygen
%endif
BuildRequires: flex
BuildRequires: sed
BuildRequires: graphviz
BuildRequires: libstdc++-devel
BuildRequires: libtool
BuildRequires: zip
BuildRequires: libffi-devel
BuildRequires: chrpath

%description
LVM is a robust system, particularly well suited for developing new mid-level
language-independent analyses and optimizations of all sorts, including those
that require  extensive interprocedural analysis. LLVM is also a great target
for front-end development for conventional or research programming languages,
including those which require compile-time, link-time, or run-time optimization 
for effective implementation, proper tail calls or garbage collection. 

%files
%defattr(-,root,root,-)
%doc LICENSE.TXT
%{_bindir}/bugpoint
%{_bindir}/llc
%{_bindir}/lli
%{_bindir}/opt
%{_bindir}/llvm-ar
%{_bindir}/llvm-as
%{_bindir}/llvm-bcanalyzer
%{_bindir}/llvm-diff
%{_bindir}/llvm-dis
%{_bindir}/llvm-extract
%{_bindir}/llvm-ld
%{_bindir}/llvm-link
%{_bindir}/llvm-mc
%{_bindir}/llvm-nm
%{_bindir}/llvm-objdump
%{_bindir}/llvm-prof
%{_bindir}/llvm-ranlib
%{_bindir}/llvm-readobj
%{_bindir}/llvm-stub
%{_bindir}/llvm-cov
%{_bindir}/llvm-dwarfdump
%{_bindir}/llvm-rtdyld
%{_bindir}/llvm-size
%{_bindir}/llvm-stress
%{_bindir}/llvm-tblgen
%{_bindir}/macho-dump
%{_mandir}/man1/bugpoint.1*
%{_mandir}/man1/l*
%{_mandir}/man1/opt.1*
%{_mandir}/man1/tblgen.1*
%{_libdir}/ocaml/*

#-----------------------------------------------------------

%define major %version
%define libname %mklibname %name %major

%package -n %libname
Summary: LLVM shared libraries
Group: System/Libraries
Conflicts: llvm < 3.0-4

%description -n %libname
Shared libraries for the LLVM compiler infrastructure. This is needed by
programs that are dynamically linked against libLLVM.

%files -n %libname
%{_libdir}/libLLVM-%major.so

#-----------------------------------------------------------

%define libname_devel %mklibname -d %name

%package -n %{libname_devel}
Summary: Development files for LLVM
Group: Development/Other
Provides: llvm-devel = %version-%release
Requires: %libname = %version-%release
Requires: %name = %version-%release
Conflicts: llvm < 3.0-7
Conflicts: %{_lib}llvm3.0 < 3.0-9

%description -n %{libname_devel}
This package contains the development files for LLVM;

%files -n %{libname_devel}
%{_bindir}/%{name}-config
%{_libdir}/libLLVM.so
%{_includedir}/%{name}
%{_includedir}/%{name}-c
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/BugpointPasses.so
%{_libdir}/%{name}/libLLVM*.a
%{_libdir}/%{name}/libLLVM*.so
%{_libdir}/%{name}/libLTO.a
%{_libdir}/%{name}/libLTO.so
%{_libdir}/%{name}/libprofile_rt.a
%{_libdir}/%{name}/libprofile_rt.so
%{_libdir}/%{name}/libllvm*.a

#-----------------------------------------------------------

%package doc
Summary: Documentation for LLVM
Group: Books/Computer books
Requires: %{name} = %{version}
BuildArch: noarch
Obsoletes: llvm-doc-devel

%description doc
Documentation for the LLVM compiler infrastructure.

%files doc
%defattr(-,root,root,-)
%doc README.txt
%doc docs/*.css
%doc docs/*.html
%doc docs/img
%doc docs/tutorial
%doc docs/ocamldoc
%doc examples
%if %{compile_apidox}
%doc docs/doxygen
%endif

#-----------------------------------------------------------

%if %{with clang}
%define clang_major %version
%define clang_libname %mklibname clang %clang_major

# TODO: %{_bindir}/clang is linked against static libclang.a, could it be
# linked against libclang.so instead, like llvm-* are against livLLVM.so?

%package -n clang
Summary:        A C language family front-end for LLVM
License:        NCSA
Group:          Development/Other
# TODO: is this requires:llvm needed, or just legacy from fedora pkg layout?
Requires:       llvm%{?_isa} = %{version}-%{release}
# clang requires gcc, clang++ requires libstdc++-devel
Requires:       gcc
Requires:       libstdc++-devel = %{gcc_version}

%description -n clang
clang: noun
    1. A loud, resonant, metallic sound.
    2. The strident call of a crane or goose.
    3. C-language family front-end toolkit.

The goal of the Clang project is to create a new C, C++, Objective C
and Objective C++ front-end for the LLVM compiler. Its tools are built
as libraries and designed to be loosely-coupled and extensible.

%files -n clang
%doc clang-docs/*
%{_bindir}/clang*
%{_bindir}/c-index-test
%{_prefix}/lib/clang
%doc %{_mandir}/man1/clang.1.*

%package -n %clang_libname
Summary: Shared library for clang
Group: System/Libraries

%description -n %clang_libname
Shared libraries for the clang compiler. This is needed by
programs that are dynamically linked against libclang.

%files -n %clang_libname
%{_libdir}/libclang-%clang_major.so

#-----------------------------------------------------------

%define clang_libname_devel %mklibname -d clang

%package -n %{clang_libname_devel}
Summary:        Development files for clang
Group:          Development/Other
Requires:	%clang_libname = %version-%release
Provides:	clang-devel = %version-%release
Conflicts:	llvm-devel < 3.1
Obsoletes:	clang-devel < 3.1

%description -n %{clang_libname_devel}
This package contains header files and libraries needed for using
libclang.

%files -n %{clang_libname_devel}
%{_includedir}/clang
%{_includedir}/clang-c
%{_libdir}/libclang.so
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/libclang*.a
%{_libdir}/%{name}/libclang*.so

%package -n clang-analyzer
Summary:        A source code analysis framework
License:        NCSA
Group:          Development/Other
Requires:       clang%{?_isa} = %{version}-%{release}
# not picked up automatically since files are currently not instaled
# in standard Python hierarchies yet
Requires:       python

%description -n clang-analyzer
The Clang Static Analyzer consists of both a source code analysis
framework and a standalone tool that finds bugs in C and Objective-C
programs. The standalone tool is invoked from the command-line, and is
intended to run in tandem with a build of a project or code base.

%files -n clang-analyzer
%{_bindir}/scan-build
%{_bindir}/scan-view
%{_libdir}/clang-analyzer


%package -n clang-doc
Summary:        Documentation for Clang
Group:          Books/Computer books
BuildArch:      noarch
Requires:       %{name} = %{version}-%{release}

%description -n clang-doc
Documentation for the Clang compiler front-end.

%files -n clang-doc
%doc clang-docs-full/*

%endif

#-----------------------------------------------------------

%prep
%setup -qn %{name}-%{version}.src %{?with_clang:-a1}
rm -rf tools/clang
%if %{with clang}
mv clang-%{version}%{?prerel}.src tools/clang
cd tools/clang
%patch0 -p1
%patch1 -p1
cd -
%endif

%build

# Build with gcc/g++, not clang if it happens to be installed
# (blino) clang < 3.1 does not handle system headers from gcc 4.7
# http://llvm.org/bugs/show_bug.cgi?id=11916
export CC=gcc
export CXX=g++

%configure2_5x \
	--libdir=%{_libdir}/%{name} \
	--datadir=%{_datadir}/%{name} \
	--enable-shared \
	--enable-jit \
	--enable-libffi \
	--enable-optimized \
	--enable-targets=host-only \
	--disable-expensive-checks \
	--enable-debug-runtime \
	--disable-assertions \
	--enable-threads \
%if %{compile_apidox}
	--enable-doxygen
%endif

# FIXME file this
# configure does not properly specify libdir
sed -i 's|(PROJ_prefix)/lib|(PROJ_prefix)/%{_lib}/%{name}|g' Makefile.config

# FIXME upstream need to fix this
# llvm-config.cpp hardcodes lib in it
sed -i 's|ActiveLibDir = ActivePrefix + "/lib"|ActiveLibDir = ActivePrefix + "/%{_lib}/%{name}"|g' tools/llvm-config/llvm-config.cpp

%make

%install
cp bindings/ocaml/llvm/META.llvm bindings/ocaml/llvm/Release/
%makeinstall_std \
	KEEP_SYMBOLS=1 \
	PROJ_docsdir=%{_docdir}/%{name} \
	PROJ_etcdir=%{_sysconfdir}/%{name} \
	PROJ_libdir=%{_libdir}/%{name}

# Invalid dir 
rm -rf %buildroot%_bindir/.dir

# wrong rpath entries (Anssi 11/2011)
file %{buildroot}/%{_bindir}/* | awk -F: '$2~/ELF/{print $1}' | xargs -r chrpath -d
file %{buildroot}/%{_libdir}/llvm/*.so | awk -F: '$2~/ELF/{print $1}' | xargs -r chrpath -d

# move shared library to standard library path and add devel symlink (Anssi 11/2011)
mv %{buildroot}%{_libdir}/llvm/libLLVM-%major.so %{buildroot}%{_libdir}
ln -s libLLVM-%major.so %{buildroot}%{_libdir}/libLLVM.so
# Also, create shared library symlinks corresponding to all the static library
# names, so that using e.g. "-lLLVMBitReader" will cause the binary to be linked
# against the shared library instead of static library by default. (Anssi 08/2012)
for staticlib in %{buildroot}%{_libdir}/llvm/libLLVM*.a; do
	sharedlib="${staticlib%.a}.so"
	[ -e "$sharedlib" ] && exit 1
	ln -s ../libLLVM.so "$sharedlib"
done

%if %with clang
# Versionize libclang.so (patch0 makes the same change to soname) and move it to standard path.
mv %{buildroot}%{_libdir}/llvm/libclang.so %{buildroot}%{_libdir}/libclang-%version.so
ln -s libclang-%clang_major.so %{buildroot}%{_libdir}/libclang.so
ln -s ../libclang.so %{buildroot}%{_libdir}/llvm/libclang.so

# NOTE: We don't create devel symlinks for the libclang.so for libclang*.a libraries
# like for libLLVM above, because libclang.so actually exports much less symbols
# - some are not linked in (tools/libclang/Makefile) and others are restricted
# by tools/libclang/libclang.exports. - Anssi 09/2012
%endif

# Since the static libraries are very huge, strip them of debug symbols as well
# (Anssi 08/2012)
strip --strip-debug %{buildroot}%{_libdir}/llvm/*.a

%if %{with clang}

# Static analyzer not installed by default:
# http://clang-analyzer.llvm.org/installation#OtherPlatforms
mkdir -p %{buildroot}%{_libdir}/clang-analyzer
# create launchers
for f in scan-{build,view}; do
  ln -s %{_libdir}/clang-analyzer/$f/$f %{buildroot}%{_bindir}/$f
done

(cd tools/clang/tools && cp -pr scan-{build,view} \
 %{buildroot}%{_libdir}/clang-analyzer/)

# And prepare Clang documentation
#
rm -rf clang-docs
mkdir clang-docs
for f in LICENSE.TXT NOTES.txt README.txt; do # TODO.txt; do
  ln tools/clang/$f clang-docs/
done
rm -rf clang-docs-full
cp -al tools/clang/docs clang-docs-full
rm -rf clang-docs-full/{doxygen*,Makefile*,*.graffle,tools}

%endif

# Get rid of erroneously installed example files.
rm %{buildroot}%{_libdir}/%{name}/LLVMHello.so


%changelog
* Fri Oct 05 2012 Bernhard Rosenkraenzer <bero@bero.eu> 3.1-2mdv2012.0
+ Revision: 818433
- Bump release, needs to be sent through the build system again

* Mon Sep 17 2012 Alexander Khrukin <akhrukin@mandriva.org> 3.1-1
+ Revision: 817008
- version update 3.1 and mga sync

* Tue Jan 10 2012 Paulo Andrade <pcpa@mandriva.com.br> 3.0-4
+ Revision: 759605
- Rename library packages to match soname.

* Sun Jan 08 2012 Funda Wang <fwang@mandriva.org> 3.0-3
+ Revision: 758660
- force to disable assertion

* Thu Dec 29 2011 Paulo Andrade <pcpa@mandriva.com.br> 3.0-2
+ Revision: 748201
- Build versioned shared libraries and split packages for easier upgrades.

* Wed Dec 07 2011 Paulo Andrade <pcpa@mandriva.com.br> 3.0-1
+ Revision: 738726
- Update to llvm 3.0 and enable clang 3.0.

* Wed May 18 2011 Funda Wang <fwang@mandriva.org> 2.9-3
+ Revision: 675990
- rebuild
- add upstream patch to deal with -std=c++0x flag used in lightspark (upstream bug#9869)

* Tue Apr 26 2011 Funda Wang <fwang@mandriva.org> 2.9-1
+ Revision: 659078
- update file list
- new version 2.9

* Sun Dec 26 2010 Funda Wang <fwang@mandriva.org> 2.8-3mdv2011.0
+ Revision: 625147
- fix config script

* Sun Dec 26 2010 Funda Wang <fwang@mandriva.org> 2.8-2mdv2011.0
+ Revision: 625136
- add upstream patch to fix downstream program compiling
- fix linking problem when building with cmake

* Sun Oct 24 2010 Funda Wang <fwang@mandriva.org> 2.8-1mdv2011.0
+ Revision: 588692
- update file list
- new version 2.8

* Sat Oct 02 2010 Anssi Hannula <anssi@mandriva.org> 2.7-3mdv2011.0
+ Revision: 582535
- rebuild for new ocaml
- fix license tag to adhere policy

* Sat Sep 04 2010 Emmanuel Andry <eandry@mandriva.org> 2.7-2mdv2011.0
+ Revision: 575900
- rebuild

* Tue Jul 27 2010 John Balcaen <mikala@mandriva.org> 2.7-1mdv2011.0
+ Revision: 561372
- Update to 2.7
- fix %%files list
- fix %%makeinstall_std installation path

* Thu Jan 28 2010 Funda Wang <fwang@mandriva.org> 2.6-2mdv2010.1
+ Revision: 497583
- rebuild for new ocaml

* Tue Nov 24 2009 Funda Wang <fwang@mandriva.org> 2.6-1mdv2010.1
+ Revision: 469643
- update file list
- new version 2.6

* Sun Sep 27 2009 Olivier Blin <blino@mandriva.org> 2.5-8mdv2010.0
+ Revision: 450115
- make the configure stuff know about mips and mipsel
  (rediffed patch from Arnaud Patard)

  + Nicolas Lécureuil <nlecureuil@mandriva.com>
    - Fix macro

* Thu Sep 17 2009 Nicolas Lécureuil <nlecureuil@mandriva.com> 2.5-7mdv2010.0
+ Revision: 443995
- Fix build
- Fix build on 64 bit arch

* Thu Sep 17 2009 Nicolas Lécureuil <nlecureuil@mandriva.com> 2.5-6mdv2010.0
+ Revision: 443921
- Do not use Pic on i586

* Wed Sep 16 2009 Nicolas Lécureuil <nlecureuil@mandriva.com> 2.5-5mdv2010.0
+ Revision: 443651
- Really disable assertions

* Wed Sep 16 2009 Nicolas Lécureuil <nlecureuil@mandriva.com> 2.5-4mdv2010.0
+ Revision: 443345
- Do not build with assert ( openctl does not support it)

* Wed Jul 29 2009 Funda Wang <fwang@mandriva.org> 2.5-3mdv2010.0
+ Revision: 403563
- bump rel
- fix build with gcc4.4
- rebuild for new ocaml

* Wed May 27 2009 Eugeni Dodonov <eugeni@mandriva.com> 2.5-1mdv2010.0
+ Revision: 380082
- Updated to 2.5.
  Dropped P0 and P2 (integrated upstream).

* Fri Apr 10 2009 Funda Wang <fwang@mandriva.org> 2.3-5mdv2009.1
+ Revision: 365764
- fix str fmt

* Thu Sep 18 2008 Helio Chissini de Castro <helio@mandriva.com> 2.3-5mdv2009.0
+ Revision: 285655
- Yeah, bad idea let devel doc enabled by default

* Tue Sep 16 2008 Helio Chissini de Castro <helio@mandriva.com> 2.3-4mdv2009.0
+ Revision: 285358
- Simplify package which is a compiler and never was suppose to be splitted at begin. Doc is integrated in one only package too

* Fri Jul 04 2008 Helio Chissini de Castro <helio@mandriva.com> 2.3-2mdv2009.0
+ Revision: 231722
- Fix llvm-config to match lib dir

* Tue Jul 01 2008 Helio Chissini de Castro <helio@mandriva.com> 2.3-1mdv2009.0
+ Revision: 230557
- import llvm

